#!/usr/bin/env python
"""
U2FsdGVkX19cV2plTMSqbpdFaJuEChbUka1NX+X4qeSMMFuTquH1Wd+vL6e2Hf4H
CAvb+R3+MDOCjdOyhgS3DNleCDqZRRJorq7lehTQzbA/pY9TMQBMyUnm63wUOjd/
EKVAggOM+jODpiFn0+FN4le8LUoOErVSZSR7L4gka/7L69wT82r90gq12H1mFNns
judHXORdh3Iud6J8UQHqSZUSFRfnd1IetIIySIIVxxCPE0DY2CS7Y/TjmEiHduzm
l4TiHrn9MlQOJY21PIjswrB9EIS/fsQsNV09BIBh6cJTu2jPULRjOpVGw+liyvkJ
I6HynL0O7XEVC61l2wiQJA==
"""

"""Shared utility functions.
"""
import collections

EXCLUDE = ['preamble', 'receiver_address', 'source_address', 'data_length', 'flag', 'command', 'payload', 'crc']

from construct import Container

# Power Section Slaves
PsbpSlave = collections.namedtuple('FlashingState', 'ONE TWO THREE')._make(range(0x40, 0x40 + 3))


class PsbpMode(object):
    OFF = 0
    PHASE = int(1)
    PULSE = int(2)


class PsbpFlag(object):
    NULL = 0
    ACK = (1 << 0)
    READY = (1 << 1)
    HC_DEFECT = (1 << 2)
    OVER_CURRENT = (1 << 3)
    ACK_REQUEST = (1 << 4)
    OVER_TEMPERATURE = (1 << 5)
    RELAY_ON = (1 << 6)
    ERROR = (1 << 7)


def exclude_fields(obj, exclude=EXCLUDE):
    """
    Return dict of object without parent attrs.
    """
    return dict([(k, getattr(obj, k)) for k in obj.__slots__ if k not in exclude])


def containerize(coll):
    """Walk attribute fields passed from an PSBP message and convert to
    Containers where appropriate. Needed for Construct proper
    serialization.

    Parameters
    ----------
    coll : dict

    """
    if isinstance(coll, Container):
        [setattr(coll, k, containerize(v)) for (k, v) in coll.iteritems()]
        return coll
    elif isinstance(coll, dict):
        return containerize(Container(**coll))
    elif isinstance(coll, list):
        for j, i in enumerate(coll):
            if isinstance(i, dict):
                coll[j] = containerize(Container(**i))
        return coll
    else:
        return coll


def fmt_repr(obj):
    """Print a orphaned string representation of an object without the
    clutter of its parent object.

    """
    items = ["%s = %r" % (k, v) for k, v in exclude_fields(obj).items()]
    return "<%s: {%s}>" % (obj.__class__.__name__, ', '.join(items))
